@@ -12,7 +12,11 @@ use Log::Log4perl::Config;
 use Log::Dispatch::Screen;
 use Log::Log4perl::Appender;
 
-our $VERSION = '0.21';
+our $VERSION = '0.22';
+
+   # set this to '1' if you're using a wrapper
+   # around Log::Log4perl
+our $caller_depth = 0;
 
 ##################################################
 sub import {
@@ -905,6 +909,8 @@ for them to tune the logging to their needs.
 
 =head1 Cool Tricks
 
+=head2 Shortcuts
+
 When getting an instance of a logger, instead of saying
 
     use Log::Log4perl;
@@ -916,12 +922,48 @@ C<Log::Log4perl> into the current namespace:
     use Log::Log4perl qw(get_logger);
     my $logger = get_logger();
 
+=head2 Alternative initialization
+
+Instead of having C<init()> read in a configuration file, you can 
+also pass in a reference to a string, containing the content of
+the file:
+
+    Log::Log4perl->init( \$config_text );
+
+Also, if you've got the C<name=value> pairs of the configuration in
+a hash, you can just as well initialized C<Log::Log4perl> with
+a reference to it:
+
+    my %key_value_pairs = (
+        "log4j.rootLogger"       => "error, LOGFILE",
+        "log4j.appender.LOGFILE" => "Log::Dispatch::File",
+        ...
+    );
+
+    Log::Log4perl->init( \%key_value_pairs );
+
 =head1 How about Log::Dispatch::Config?
 
 Yeah, I've seen it. I like it, but I think it is too dependent
 on defining everything in a configuration file.
 I've designed C<Log::Log4perl> to be more flexible.
 
+=head1 Using Log::Log4perl from wrapper classes
+
+If you don't use C<Log::Log4perl> as described above, 
+but from a wrapper class (like your own Logging class which in turn uses
+C<Log::Log4perl>),
+the pattern layout will generate wrong data for %F, %C, %L and the like.
+Reason for this is that C<Log::Log4perl>'s loggers assume a static
+caller depth to the application that's using them. If you're using
+one (or more) wrapper classes, C<Log::Log4perl> will indicate where
+your logger classes called the loggers, not where your application
+called your wrapper, which is probably what you want in this case.
+But don't dispair, there's a solution: Just increase the value
+of C<$Log::Log4perl::caller_depth> (defaults to 0) by one for every
+wrapper that's in between your application and C<Log::Log4perl>,
+then C<Log::Log4perl> will compensate for the difference.
+
 =head1 INSTALLATION
 
 C<Log::Log4perl> needs C<Log::Dispatch> (2.00 or better) and
@@ -972,17 +1014,12 @@ log4perl-devel@lists.sourceforge.net
 
 =head1 AUTHORS
 
-=over 4
+    Mike Schilli <m@perlmeister.com>
+    Kevin Goess <cpan@goess.org>
 
-=item *
+    Contributors:
 
-Mike Schilli, m@perlmeister.com
-
-=item *
-
-Kevin Goess, cpan@goess.org
-
-=back
+    Chris R. Donnelly <cdonnelly@digitalmotorworks.com>
 
 =head1 COPYRIGHT AND LICENSE
 
