@@ -6,6 +6,9 @@ use 5.006;
 use strict;
 use warnings;
 
+use Log::Log4perl::Level;
+use constant DEBUG => 0;
+
 our $unique_counter = 0;
 
 ##################################################
@@ -51,6 +54,7 @@ sub new {
                  appender => $appender,
                  name     => $params{name},
                  layout   => undef,
+                 level    => $DEBUG,
                };
     
     bless $self, $class;
@@ -58,11 +62,32 @@ sub new {
     return $self;
 }
 
+##################################################
+sub threshold { # Set/Get the appender threshold
+##################################################
+    my ($self, $level) = @_;
+
+    print "Setting threshold to $level\n" if DEBUG;
+
+    if(defined $level) {
+        $self->{level} = $level;
+    }
+
+    return $self->{level};
+}
+
 ##################################################
 sub log { # Relay this call to Log::Dispatch::Whatever
 ##################################################
     my ($self, $p, $category, $level) = @_;
 
+    # Check if the appender has a last-minute veto in form
+    # of an "appender threshold"
+    if($self->{level} < $
+                        Log::Log4perl::Level::PRIORITY{$level}) {
+        return;
+    }
+
     $self->{layout} || $self->layout();  #set to default if not already
                                          #can this be moved?
 
@@ -71,7 +96,7 @@ sub log { # Relay this call to Log::Dispatch::Whatever
     $p->{message} = $self->{layout}->render($p->{message}, 
                                             $category,
                                             $level,
-                                            3,
+                                            3 + $Log::Log4perl::caller_depth,
                                             );
     return $self->{appender}->log(%$p);
 }
